/*
 * Storage 一个支持过期特性的缓存类
 */

/*
 * 获取缓存
 * @param { String } key 本地缓存中的指定的 key
 */
function get(key) {
    let data = wx.getStorageSync(key)
    if (!data) {
        return null
    }

    data = JSON.parse(data)
    // 已过期
    if (validExpire(new Date().getTime(), data.writeTime, data.durationTime)) {
        remove(key)
        return null
    } else {
        return data.data
    }
}

/*
 * 将数据存储在本地缓存中指定的 key 中，会覆盖掉原来该 key 对应的内容
 * @param { String } key 本地缓存中的指定的 key
 * @param { Any } data 需要存储的内容，只支持原生类型、及能够通过 JSON.stringify 序列化的对象
 * @param { Number } duration 有效时长，单位毫秒
 */
function set(key, data, duration) {
    const obj = {
        data: data, // 存储数据
        writeTime: new Date().getTime(), // 写入时间
        durationTime: duration // 有效时长 单位毫秒
    }
    wx.setStorageSync(key, JSON.stringify(obj))
}

/*
 * 从本地缓存中异步移除指定 key
 * @param { String } key 本地缓存中的指定的 key
 */
function remove(key) {
    wx.removeStorageSync(key)
}

/*
 * 清理本地数据缓存
 */
function clear() {
    wx.clearStorageSync()
}

/*
 * 验证是否过期
 * @param { Number } nowTime 当前时间
 * @param { Number } writeTime 缓存写入时间
 * @param { Number } expireTime 缓存有效时长
 */
function validExpire(nowTime, writeTime, expireTime) {
    if (!expireTime && expireTime !== 0) {
        return false
    }
    if (nowTime > writeTime + expireTime) {
        return true
    }
    return false
}

export default {
    /*
     * 获取缓存
     * @param { String } key 本地缓存中的指定的 key
     */
    get,

    /*
     * 将数据存储在本地缓存中指定的 key 中，会覆盖掉原来该 key 对应的内容
     * @param { String } key 本地缓存中的指定的 key
     * @param { Any } data 需要存储的内容，只支持原生类型、及能够通过 JSON.stringify 序列化的对象
     * @param { Number } duration 有效时长，单位毫秒
     */
    set,

    /*
     * 从本地缓存中异步移除指定 key
     * @param { String } key 本地缓存中的指定的 key
     */
    remove,

    /*
     * 清理本地数据缓存
     */
    clear,

    /*
     * 验证是否过期
     * @param { Number } nowTime 当前时间
     * @param { Number } writeTime 缓存写入时间
     * @param { Number } expireTime 缓存有效时长
     */
    validExpire
}